/* Author: R.Harmsen
   Copyright 1989-2000 R.Harmsen
*/

/* Program for automatic conversion of umlauts.
 * Sequences like "ae",	"ue", "ss" etc.	are converted
 *   to	ISO-8859-! characters.
 *
 * The program knows about the rules and exceptions
 * of the German language.
 *
 * See also: directory /ruud/umlaut for an older version.
 */

#include <stdio.h>
#include "ae-acc.h"

static int to_8859 (int c);
static int umlaut_ok (char *cp);
static int u_exception (char *cp);
static int ring_s_ok (char *cp);
static int streq (char *sub, char *str);


int acc_to_ae (char *out, char *in)
{
   char *p, *q;

   for (p = in, q = out; *p; p++, q++)
   {
      int add_e = 0;
      int add_s = 0;

      if (*p == '\xe4') /* a-umlaut   */
      {
         add_e = 1;
         *q = 'a';
      }
      else if (*p == '\xf6') /* o-umlaut   */
      {
         add_e = 1;
         *q = 'o';
      }
      else if (*p == '\xfc') /* u-umlaut   */
      {
         add_e = 1;
         *q = 'u';
      }
      else if (*p == '\xc4') /* A-umlaut   */
      {
         add_e = 1;
         *q = 'A';
      }
      else if (*p == '\xd6') /* O-umlaut   */
      {
         add_e = 1;
         *q = 'O';
      }
      else if (*p == '\xdc') /* U-umlaut   */
      {
         add_e = 1;
         *q = 'U';
      }
      else if (*p == '\xdf') /* Scharfes s */
      {
         add_s = 1;
         *q = 's';
      }
      else
      {
         *q = *p;
      }

      if (add_e)
      {
         *++q = 'e';
      }
      if (add_s)
      {
         *++q = 's';
      }
   }
   *q = '\0';

   return 0;

}

int ae_to_acc (char *in)
{
   char *p;

   for (p = in + 1; *p; p++)
   {
      if (*p == 'e' && umlaut_ok(p))
      {
         *p = '^';
      }
      else if (*p == 's' && ring_s_ok(p))
      {
         *p = '^';
      }
   }

   return 0;
}

int acc_to_iso (char *out, char *in)
{
   char *p, *q;

   /* First character, handled separately because ok function
      test previous. */
   *out = *in;

   for (p = in + 1, q = out + 1; *p; p++, q++)
   {
      if (*p == '^' &&
         (
            p[-1] == 'a' ||
            p[-1] == 'o' ||
            p[-1] == 'u' ||
            p[-1] == 'A' ||
            p[-1] == 'O' ||
            p[-1] == 'U' ||
            p[-1] == 's'
         ))
      {
         q--;
         *q = to_8859(*q);
      }
      else
      {
         *q = *p;
      }
   }
   *q = '\0';

   return 0;
}

static int to_8859 (int c)
{
   switch (c)
   {
      case 'a':
         return 0xe4; /* a-umlaut   */
      case 'o':
         return 0xf6; /* o-umlaut   */
      case 'u':
         return 0xfc; /* u-umlaut   */
      case 'A':
         return 0xc4; /* A-umlaut   */
      case 'O':
         return 0xd6; /* O-umlaut   */
      case 'U':
         return 0xdc; /* U-umlaut   */
      case 's':
         return 0xdf; /* Scharfes s */
      default:
         return c;
   }
}

static int umlaut_ok (char *cp)
{
   char *p;

   if (streq("a", p=cp-1) ||
       streq("o", p) ||
       streq("A", p) ||
       streq("O", p) ||
       streq("U", p) )
       return 1;
   else if (streq("u", p) && !u_exception(cp))
      return 1;
   else
      return 0;
}

static int u_exception (char *cp)
{
    char *p;

    if (*(p=cp-2) == 'a' ||
	*p == 'e' ||
	*(cp+1)	== 'i' ||
	*p == 'q' ||
	*p == 'Q' )
	return 1;
    else if (streq("ll", cp+1) &&
	(streq ("d", p)	|| streq ("t",p) || streq ("n",p) || streq ("s",p)))
	return 1;
    else if ((streq ("z", p) ||	streq ("Z", p))	&& streq("r", cp+1))
	return 1;
    else if (streq ("val", p=cp-4) ||
	     streq ("Val", p) ||
	     streq ("tin", p))
	return 1;
    else
	return 0;
}

static int ring_s_ok (char *cp)
{
    char *p;

    if (*(cp-1)	!='s')
	return 0;
    else if ((*(cp-2) == 'a' ||	*(cp-2)	== 'i')	&&
	*(p=cp+1) != 'e' && *p != 'i' && *p != 'u' && *p != 'a'	&&
	(*(p=cp-3) == 'm' ||
	*p == 'd' ||
       (*p == 'p' && !(streq("word", cp+1) || streq("wd", cp+1))) ||
	*p == 'f' ||
	*p == 'M' ||
	*p == 'D' ||
       (*p == 'P' && !(streq("word", cp+1) || streq("wd", cp+1))) ||
	*p == 'F' ) )
	return 1;
    else if (streq ("ie", p=cp-3) || streq ("ei", p) ||
	(streq ("o^", p) && ! streq ("ch", cp+1)) ||
	(streq ("a^", p) && !(*(cp-4)=='l' && *(cp+1) == 'i')))
	return 1;
    else if (streq ("gro", p=cp-4) || streq ("Gro",p) ||
	(streq ("wu^", p) && *(cp+1) ==	't') ||
	(streq ("mu^", p) && *(cp+1) ==	't') ||
	(streq ("Wu^", p) && *(cp+1) ==	't') ||
	(streq ("Mu^", p) && *(cp+1) ==	't') ||
	(streq ("ze", cp-3) && *(cp+1) != 'e'))
	return 1;
    else if (*(cp-2) ==	'u')
    {
	if ((*(p=cp-3) == 'a' || *p == 'A' ||
	    streq ("a^u", p=cp-4) || streq ("A^u", p)) &&
		(streq ("en", p=cp+1) || streq ("er", p)))
	    return 1;
	else if	(*(p=cp-3) == 'a' || *p	=='A' ||
	    (*(p=cp+1) == 'e' || *p=='i'||*p=='u'))
	    return 0;
	else
	    return 1;
     }
     else
	return 0;
}

static int streq (char *sub, char *str)
{
    int ret = 1;

    while (*sub	!= '\0')
    {
	if (*sub++ != *str++)
	{
	    ret	= 0;
	    break;
	}
    }
    return ret;
}
